package route

import (
	"git.hilo.cn/hilo-common/domain"
	"git.hilo.cn/hilo-common/mylogrus"
	"github.com/gin-gonic/gin"
	ginSwagger "github.com/swaggo/gin-swagger"
	"github.com/swaggo/gin-swagger/swaggerFiles"
	"hilo-group/_const/enum/mgr_e"
	_ "hilo-group/docs"
	"hilo-group/domain/model/mgr_m"
	"hilo-group/route/charge_r"
	"hilo-group/route/group_power_r"
	"hilo-group/route/group_r"
)

func InitRouter() *gin.Engine {
	var r = gin.Default()

	r.GET("/group-swagger/*any", ginSwagger.WrapHandler(swaggerFiles.Handler))

	needLogin := r.Group("")
	needLogin.Use(ExceptionHandle, LoggerHandle, JWTApiHandle)
	v1 := needLogin.Group("/v1")
	imGroup := v1.Group("/imGroup")
	{
		imGroup.GET("/popular", wrapper(group_r.GetPopularGroups))
		imGroup.GET("/latest", wrapper(group_r.GetLatestGroups))
		imGroup.GET("/recommended", wrapper(group_r.GetRecommendGroup))
		//
		imGroup.POST("/group", wrapper(group_r.CreateGroup))
		imGroup.DELETE("/group/:groupId", wrapper(group_r.DestroyGroup))
		imGroup.GET("/group/:code", wrapper(group_r.GetGroupInfo))
		imGroup.GET("/detail/:groupId", wrapper(group_r.GetGroupDetail))
		imGroup.PUT("/group/:groupId", wrapper(group_r.ModifyGroupInfo))
		imGroup.GET("/search/:code", wrapper(group_r.SearchGroup))
		imGroup.DELETE("/member/:groupId", wrapper(group_r.LeaveGroup))
		imGroup.PUT("/permanent/:groupId", wrapper(group_r.AddPermanentMember))
		imGroup.DELETE("/permanent/:groupId", wrapper(group_r.RemovePermanentMember))
		imGroup.GET("/permanent/:groupId", wrapper(group_r.GetPermanentMember))
		imGroup.GET("/myRecent", wrapper(group_r.GetRecentGroup))
		imGroup.GET("/myPermanent", wrapper(group_r.GetMyGroup))
		//
		imGroup.GET("/visitors/:groupId", wrapper(group_r.GetGroupVisitors))
		imGroup.GET("/ownPublicGroup/:userExternalId", wrapper(group_r.GetOwnPublicGroup))
		//// 2.19的新接口
		imGroup.GET("/ownGroup", wrapper(group_r.GetOwnGroup))
		imGroup.GET("/theirGroup/:userExternalId", wrapper(group_r.GetTheirGroups))
		imGroup.PUT("/pluginReady/:groupId", wrapper(group_r.PluginReady))
		imGroup.GET("/roomInfo/:groupId", wrapper(group_r.GetRoomInfo))
		//
		imGroup.GET("/password/:groupId", wrapper(group_r.GetGroupPassword))
		imGroup.GET("/role/:groupId", wrapper(group_r.GetGroupRole))
		imGroup.PUT("/role/:groupId", wrapper(group_r.SetGroupRole))
		imGroup.POST("/role/accept", wrapper(group_r.AcceptMemberInvite))
		imGroup.GET("/admin/:groupId", wrapper(group_r.GetGroupAdmin))
		//
		imGroup.PUT("/blacklist/:groupId", wrapper(group_r.AddGroupBlacklist))
		imGroup.DELETE("/blacklist/:groupId", wrapper(group_r.RemoveGroupBlacklist))
		imGroup.GET("/blacklist/:groupId", wrapper(group_r.GetGroupBlacklist))
		imGroup.PUT("/kick/:groupId", wrapper(group_r.KickGroupMembers))
		//
		imGroup.PUT("/allGroupMsg", wrapper(group_r.SendTextMsg))
		//
		imGroup.PUT("/welcomeText/:groupId", wrapper(group_r.SetWelcomeText))
		imGroup.GET("/support/page/:groupId", wrapper(group_r.GetSupportPage))
		imGroup.GET("/support/detail", wrapper(group_r.GetSupportDetail))
		imGroup.GET("/support/award/:groupId", wrapper(group_r.TryAddSupporter))
		imGroup.POST("/support/award/:groupId", wrapper(group_r.TakeSupportAward))
		//
		//// 操作类，普通用户不用
		imGroup.PUT("/memberLimit", wrapper(group_r.SetGroupMemberLimit))
		imGroup.PUT("/info/reset", wrapper(group_r.ResetGroupInfo), OperationLog)
		imGroup.PUT("/upgrade", wrapper(group_r.UpgradeGroup))
		imGroup.PUT("/downgrade", wrapper(group_r.DowngradeGroup))
		//
		//imGroup.GET("/mic/all", wrapper(GroupMicAllInfoFive))
		//imGroup.GET("/mic/all/type", wrapper(GroupMicAllInfoTen))
		//imGroup.GET("/mic/all/type/new", wrapper(GroupMicAllInfoType))
		//imGroup.PUT("/mic/num", wrapper(GroupMicNumChange))
		//imGroup.GET("/mic/num", wrapper(GroupMicNum))
		//imGroup.POST("/mic/emoji/msg", wrapper(GroupSendMicSystemMsg))
		//imGroup.POST("/mic/in/invite/dialog", wrapper(GroupMicInInviteDialog))
		//imGroup.POST("/mic/task/invite/dialog", wrapper(GroupMicTaskInviteDialog))
		//imGroup.POST("/mic/in", LogRequestTime, wrapper(GroupMicIn))
		//imGroup.POST("/mic/invite", LogRequestTime, wrapper(GroupMicInvite))
		//imGroup.POST("/mic/leave", LogRequestTime, wrapper(GroupMicLeave))
		//imGroup.POST("/mic/lock", wrapper(GroupMicLock))
		//imGroup.POST("/mic/unlock", wrapper(GroupMicUnLock))
		//imGroup.POST("/mic/speech/open", wrapper(GroupMicSpeechOpen))
		//imGroup.POST("/mic/speech/close", wrapper(GroupMicSpeechClose))
		//imGroup.POST("/mic/mute", wrapper(GroupMicMute))
		//imGroup.POST("/mic/unmute", wrapper(GroupMicUnmute))
		//imGroup.PUT("/in", LogRequestTime, wrapper(GroupIn))
		//imGroup.POST("/leave", wrapper(GroupLeave))
		//imGroup.POST("/kick", wrapper(GroupKick))
		//imGroup.PUT("/user/msg/status", wrapper(GroupUserMsg))
		//imGroup.POST("/report", wrapper(GroupReport))
		//imGroup.GET("/banner/list", wrapper(GroupBannerList))
		//imGroup.GET("/roomBanners", wrapper(RoomBannerList))
		//imGroup.PUT("/roomBanners", wrapper(NotifyRoomBannerListChange))
		//imGroup.POST("/mic/gift", wrapper(GroupMicGift))
		//imGroup.POST("/mic/mass", wrapper(GroupMicMass))
		//imGroup.POST("/mgr/mass", wrapper(GroupMgrMass))
		//imGroup.POST("/mgr/clearScreen", wrapper(GroupMgrClearScreen))
		//imGroup.GET("/online/users", wrapper(GroupInUsers))
		//imGroup.GET("/online/users/new", wrapper(GroupInUserNew))
		//imGroup.GET("/country", wrapper(GetGroupByCountry))
		//imGroup.GET("/country/prior", wrapper(GroupountryPrior))
		//
		//imGroup.POST("/theme/custom", wrapper(GroupThemeAdd))
		//imGroup.GET("/theme/custom/config", wrapper(GroupThemeConfig))
		//imGroup.PUT("/theme/custom/using", wrapper(GroupThemeUsing))
		//imGroup.GET("/theme/custom/all", wrapper(GroupThemeValidAll))
		//
		//imGroup.GET("/medal/all", wrapper(GroupMedalAll))
		//imGroup.GET("/medal/room", wrapper(GetRoomMedal))
	}

	groupPower := v1.Group("/groupPower")
	{
		groupPower.POST("/user", wrapper(group_power_r.GroupPowerJoin))
		groupPower.DELETE("/user/:groupPowerId", wrapper(group_power_r.GroupPowerLeave))
		groupPower.GET("/title/:groupPowerId", wrapper(group_power_r.GetGroupPowerTitle))
		groupPower.GET("/group/:groupPowerId", wrapper(group_power_r.GetGroupPowerGroups))
		groupPower.GET("/team", wrapper(group_power_r.GroupPowerTeam))
		groupPower.GET("/billboard/week", wrapper(group_power_r.GroupPowerBillboardWeek))
		groupPower.GET("/billboard/owner/week", wrapper(group_power_r.GroupPowerBillboardOwnerWeek))
	}

	charge := v1.Group("/charge")
	{
		charge.GET("/share", wrapper(charge_r.CanShareChargeLink))
	}
	return r
}

// 操作日志
func OperationLog(c *gin.Context) {
	mylogrus.MyLog.Infof("%v-%v", c.Request.Method, c.FullPath())
	// 处理请求
	c.Next()
	model := domain.CreateModelNil()
	operConfig := mgr_m.GetOperationModuleUrl(model, c.Request.Method, c.FullPath())
	if operConfig.ID > 0 {
		// 需要记录操作日志
		if data, ok := c.Get(mgr_e.OperationLogKey); ok {
			if log, ok := data.(mgr_m.MgrOperationLog); ok {
				log.ModuleId = operConfig.ModuleId
				log.ModuleUrlId = operConfig.ID
				if err := mgr_m.SaveOperationLog(model, log); err != nil {
					model.Log.Error("SaveOperationLog fail:%v", err.Error())
				}
			}
		}
	}
}
